function output = administer_questionnaires(batch_number)
%%
load Personas_with_age_and_gender.mat
key % load API key
rng(batch_number); % initialize random number generator. This ensures the random ordering is different for each batch
number_of_personas_to_evaluate = 50;
counter=0;
personas_to_evaluate = rem((batch_number-1)*number_of_personas_to_evaluate+1:(batch_number)*number_of_personas_to_evaluate,2000); % batch of personas
personas_to_evaluate(personas_to_evaluate==0)=2000;

questionnaire_intro = fileread('questionnaire_intro.txt');

import matlab.net.* matlab.net.http.* matlab.net.http.io.*
uri = URI('https://api.openai.com/v1/chat/completions');

IDX = NaN(5,15,number_of_personas_to_evaluate,10);

for questionnaire = 1:5 % 5 questionnaires

    if questionnaire==1
        items = fileread('bfi_items_short.txt');
    elseif questionnaire==2
        items = fileread('bsss_items.txt');
    elseif questionnaire==3
        items = fileread('Machiavellianism_items.txt');
    elseif questionnaire==4
        items = fileread('Narcissism_items.txt');
    elseif questionnaire==5
        items = fileread('Psychopathy_items.txt');
    end

    eidx=strfind(items,newline); % end indices of items
    eidx(end+1)=length(items)+2;
    sidx=[1 eidx+1]; % start indices in item list
    p = length(eidx); % number of items

    [~,idx]=sort(rand(p,number_of_personas_to_evaluate));  % random indices for items (p items x personas)
    idx=idx';

    clear items_cell
    for item_no=1:p
        items_cell(item_no,:) = {items(sidx(item_no):eidx(item_no)-2)}; % convert items to cell format
    end
    persona_types_to_consider = 1:5;

    for persona_type=persona_types_to_consider % loop over persona types
        for pi = 1:number_of_personas_to_evaluate
            all_items_random_order=[];
            for ii=1:p % loop over p items
                item_text=char(items_cell(idx(pi,ii)));
                all_items_random_order=[all_items_random_order newline num2str(ii) '. ' item_text '.']; %#ok<AGROW>
            end

            questionnaire_intro_for_persona=strrep(questionnaire_intro,'XXXX',char(Persona(persona_type,personas_to_evaluate(pi)))); % insert persona
            prompt=char([questionnaire_intro_for_persona all_items_random_order]); % append items
            disp(newline)
            disp(newline)
            disp(prompt)
            try
                body = JSONProvider(struct('model','gpt-4-turbo','temperature',0,'n',1,'messages', {{struct('role', 'user','content', prompt')}}));
                response = RequestMessage('post', [HeaderField('Content-Type', 'application/json') HeaderField('Authorization', api_key)], body).send(uri.EncodedURI);
                output=response.Body.Data.choices.message.content;
                disp(['Counter: ' num2str(counter) ', Questionnaire: ' num2str(questionnaire) ', Persona type: ' num2str(persona_type) ', Persona number: ' num2str(pi) ' out of ' num2str(number_of_personas_to_evaluate) ', Persona number: ' num2str(personas_to_evaluate(pi)) ' out of ' num2str(size(Persona,2))])
                disp(output)
                Output{questionnaire,persona_type,pi} = output;
                IDX(questionnaire,persona_type,pi,1:p)= idx(pi,:);
            catch error
            end
            counter=counter+1;

            if counter/50 == round(counter/50) % perform a backup save for every 50 prompts
                currenttime = now;
                save(['OutputIntermediate' num2str(batch_number) '_' num2str(counter) '.mat'],'Output','prompt','idx','IDX','currenttime','personas_to_evaluate')
            end
        end % loop over personas
    end % end of persona type loop
end % end questionnaire loop

currenttime = now;
save(['OutputFinal' num2str(batch_number) '_' num2str(counter) '.mat'],'Output','prompt','idx','IDX','currenttime','personas_to_evaluate')
output=batch_number;

end