% Function that calculates Tucker's congruence coefficient between two factor loading matrices
% Lorenzo-Seva, U., & Ten Berge, J. M. (2006). Tucker's congruence coefficient as a meaningful index of factor similarity. Methodology, 2, 57-64.
% MacCallum, R. C., Widaman, K. F., Zhang, S., & Hong, S. (1999). Sample size in factor analysis. Psychological methods, 4, 84-99.
% Tucker, L. R. (1951). A method for synthesis of factor analysis studies (Personnel Research Section Report No. 984). Washington, DC: Department of the Army
% Burt, C. (1948). The factorial study of temperament traits. British Journal of Psychology, Statistical Section, 1, 178–203
% https://en.wikipedia.org/wiki/Congruence_coefficient

% The value of Tucker's Congruence Coefficient ranges from -1 to 1, 
% with 1 indicating perfect congruence, -1 indicating perfect incongruence, and 0 indicating no relationship.

function K = cc(Lambda1,Lambda2)

k = size(Lambda1,2); % number of factors
phi = NaN(k,1);

for i = 1 : k  % loop over the number of factors
    L1=Lambda1(:,i); % factor loadings of factor i
    L2=Lambda2(:,i);
    NUM = sum(L1.*L2);
    DEN = sqrt(sum(L1.^2).*sum(L2.^2));
    phi(i) = NUM/DEN;
end

K = sum(phi)/k; % take the mean value of phi across the k factors
