function Lordered = ordering(L,TL)    % input are the observed loadings (L) and the true loadings (TL)

m = size(TL,2);       % number of factors (extracted from the true loadings)
order = NaN(m,1);     % pre-allocate m x 1 vector
sgn   = NaN(m,1);     % pre-allocate m x 1 vector
CCmatrix = NaN(m,m); % m x m matrix of Tucker's Congruence Coefficients (CC)

% calculate the CC between all m*m combinations of factors
for i=1:m
    for i2=1:m
        CCmatrix(i,i2) = cc(TL(:,i),L(:,i2));
    end
end

all_CC = sort(abs(CCmatrix(:)),'descend');  % sort all absolute values of CC in descending order

for i = 1 : m*m % loop across m x m combinations
    [O, F] = find(abs(CCmatrix) == all_CC(i));  % find the i-th largest CC value from the CC matrix
    % O is the row number, F is the column number


    O = O(1); % take the first value (if they happen to be identical)
    F = F(1); % take the first value (if they happen to be identical)

    % For example, if O == 4 and F == 2, then it means that 
    % the 4th factor of the true loading matrix was matched to the 2nd factor of the observed loading matrix

    if isnan(order(O)) && ~ismember(F,order) 
        % only if factor O from the true loading matrix has not been matched yet, 
        % and only if factor F from the observed loading matrix has not been matched yet 

        order(O) = F; 
        % store the orders; so for example, if O == 4 and F == 2, then order(4) = 2
       
        sgn(O) = sign(CCmatrix(O,F)); 
        % store the sign of the CC value. 
        % so for example, if O == 4 then sgn(4) is set to 1 or -1
        % if sgn is -1, it means that the congruence is strong but the sign of the observed loadings is reversed compared to the sign of the the loadings

        CCmatrix(O,F) = NaN;  % set this CC value to NaN, so that it cannot be found again in this loop
    end
end

Lordered = L(:,order); % sort the observed factor loadings in the , correct order, so that they best match the TL
Lordered = Lordered.*repmat(sgn',size(Lordered,1),1); % correct the sign of the ordered factors as well
