clear variables;close all;clc
load Personas_with_age_and_gender.mat

TL=[-1	0	0	0	0
    0	1	0	0	0
    0	0	-1	0	0
    0	0	0	-1	0
    0	0	0	0	-1
    1	0	0	0	0
    0	-1	0	0	0
    0	0	1	0	0
    0	0	0	1	0
    0	0	0	0	1]; % true loading matrix for the BFI-10

TargetP = [10 8 9 9 9]; % number of variables in the five respective questionnaire

process_gpt_output_batches = 0; % if set to 1, then read the GPT-4 output; otherwise load the .mat file directly (this is faster speed)
if process_gpt_output_batches==1
    number_of_personas_to_evaluate = 50; % number of personas per batch
    Responses = NaN(5,5,2000,10,10); % 5 questionnaires x 15 persona types x 2000 personas x 10 reps x 10 items
    for batch_number = 1:400
        if ismember(batch_number,[1:21 23:24 27:40])
            filename = ['GPToutput\OutputFinal' num2str(batch_number) '_3750.mat'];
        else
            filename = ['GPToutput\OutputFinal' num2str(batch_number) '_1250.mat']; % persona types 1-5 are evaluated more than once
        end
        if exist(filename,"file")
            load(filename)
            rep = ceil(batch_number/40); % repetition number of persona (e.g., batch 41-80 mean that persona pi is evaluated for the 2nd time; after 40 batches of 50, all 2000 have been evaluated)
            personas_to_evaluate = rem((batch_number-1)*number_of_personas_to_evaluate+1:(batch_number)*number_of_personas_to_evaluate,2000); % batch of personas
            personas_to_evaluate(personas_to_evaluate==0)=2000;
            for questionnaire = 1:size(Output,1) % loop over questionnaires
                for persona_type = 1:5 % loop over 5 persona types
                    for pi = 1:size(Output,3) % loop over personas
                        output = Output{questionnaire,persona_type,pi};
                        idx = squeeze(IDX(questionnaire,persona_type,pi,:));
                        idx(isnan(idx))=[];
                        if length(idx)==TargetP(questionnaire)
                            split_output = str2num(output); %#ok<ST2NM>
                            if ~isempty(split_output) && length(split_output)==length(idx) % only store the responses if they are in the requested format (i.e., digits)
                                [~,o] = sort(idx); % sort the items, considering that they were presented in random order
                                Responses(questionnaire,persona_type,personas_to_evaluate(pi),rep,1:length(idx)) = split_output(o);
                            else
                                disp(['Questionnaire: ' num2str(questionnaire) ', Persona type: ' num2str(persona_type) ', Output' num2str(output) ', Number of digits: ' num2str(length(split_output))]) % display faulty outputs
                            end
                        end
                    end % end of persona loop
                end % end of persona type loop
            end % end of questionnaire loop
        end % end of 'if filename exists'
    end % end of batch number loop
    save('Responses.mat','Responses') % 5 questionnaires x 5 persona types x 2000 personas x 10 repetitions x 10 items
else
    load('Responses.mat')
end

%%  Calculate number of missing responses
disp(newline)
disp('Calculate number of missing responses')
for questionnaire = 1:size(Responses,1) % loop over 5 questionnaires
    for persona_type = 1:5 % loop over 5 persona types
        all_responses_of_questionnairepersona = squeeze(Responses(questionnaire,persona_type,:,:,1));
        all_responses_of_questionnairepersona_vector = all_responses_of_questionnairepersona(:);
        disp(['Questionnaire number:' num2str(questionnaire) ', Persona type: ' num2str(persona_type) ', Number of responses: '  ...
            num2str(length(all_responses_of_questionnairepersona_vector)) ', of which missing: ' num2str(sum(isnan(all_responses_of_questionnairepersona_vector))) ...
            ', Mean number of questionnaires per persona (out of 10): ' sprintf('%0.4f',mean(sum(~isnan(all_responses_of_questionnairepersona),2 )))])
    end
end

%%
clc
XX = NaN(5,2000,10);
for questionnaire = 1:size(Responses,1) % loop over questionnaires
    for persona_type = 1:size(Responses,2) % loop over persona types
        X = squeeze(mean(Responses(questionnaire,persona_type,:,1:10,:),4,'omitnan'));
        X=X(:,1:TargetP(questionnaire));
        invalid_idx = find(sum(isnan(X'))); % if there is 1 or more NaN, the persona is declared invalid
        valid_idx = find(sum(isnan(X'))==0); % if there is 1 or more NaN, the persona is declared invalid
        X(invalid_idx,:)=[]; % remove invalid personas
        XX(persona_type,valid_idx,1) = Age(persona_type,valid_idx);
        XX(persona_type,valid_idx,2) = Gender(persona_type,valid_idx);
        if ismember(questionnaire,1)
            [V,D] = svd(corrcoef(X)); % singular value decomposition
            D = D(1:5,1:5); % eigenvalues (retain 5)
            V = V(:,1:5); % eigenvectors (retain 5)
            L = V * sqrt(D); % the loadings are the eigenvectors multiplied by the square root of the eigenvalues
            L = rotatefactors(L,'method','promax'); % oblique rotation
            L = ordering(L,TL);

            XX(persona_type,valid_idx,3) = 6-X(:,1)  + X(:,6);    % Extraversion
            XX(persona_type,valid_idx,4) = 6-X(:,4)  + X(:,9);    % Neuroticism
            XX(persona_type,valid_idx,5) = +X(:,2)   + 6-X(:,7);  % Agreeableness
            XX(persona_type,valid_idx,6) = 6-X(:,5)  + X(:,10);   % Openness
            XX(persona_type,valid_idx,7) = 6-X(:,3)  + X(:,8);    % Conscientiousness

            if ismember(persona_type,1:5)
                disp(['Persona type: ' num2str(persona_type)])
                disp('Mean, SD, BFI-10 loadings:')
                disp([mean(X)' std(X)' L])
                disp('Mean loading strength:')
                disp(sprintf('%0.2f',mean([diag(abs(L(1:5,:)));diag(abs(L(6:10,:)))]) )) %#ok<DSPSP>
            end
        elseif ismember(questionnaire,2) % Sensation Seeking
            XX(persona_type,valid_idx,8)= sum(X,2);
        elseif ismember(questionnaire,3) % Machiavellianism
            XX(persona_type,valid_idx,9)= sum(X,2);
        elseif ismember(questionnaire,4) % Narcissism
            XX(persona_type,valid_idx,10)= sum([X(:,[1 3 4 5 7 9]) 6-X(:,[2 6 8])],2);
        elseif ismember(questionnaire,5) % Psychopathy
            XX(persona_type,valid_idx,11)= sum([X(:,[1 3 4 5 6 8 9]) 6-X(:,[2 7])],2);
        end
    end
end

% Display the construct means, standard deviations, and correlation matrix (Tables 2, S6-S9)
for persona_type = 1:5
    disp(['Persona type: ' num2str(persona_type)])
    Xa = squeeze(XX(persona_type,:,:));
    disp([mean(Xa,'omitnan')' std(Xa,'omitnan')' corr(Xa,'rows','pairwise')])
end

%% Mean response to the BFI-10 by ChatGPT and as reported by ​Lovik et al. (2017)​. The number in each marker represents the item number (see ​Table 1​).
v1 = [6-3.24   3.73    6-4.02    6-2.36    6-3.11    4.06    6-3.12   4.30   2.99   3.52]'; % from Lovik et al. (2017), Table 1.
v2 = mean(squeeze(mean(Responses(1,1,:,:,:),4)),'omitnan')';
disp(['The correlation of the item means with the corresponding item means from a previously published BFI-10 study (Lovik et al., 2017) was r = ' sprintf('%0.2f',corr(v1,v2))])
figure
colors=colormap("jet");
colors=colors(round(linspace(1,256,length(v1)))',:);
hold on;grid on;box on
clear scatter1
for i=1:10
    scatter1(i)=scatter(v1(i),v2(i),1200,'markerfacecolor','b','markeredgecolor','k');hold on;grid on %#ok<SAGROW>
    scatter1(i).MarkerFaceAlpha = .3; %#ok<SAGROW>
    text(v1(i),v2(i),num2str(i),'horizontalalignment','center','Color','w','fontweight','bold')
end
plot([1 5],[1 5],'k--','LineWidth',2)
set(gca,'xlim',[1 5],'xtick',1:0.5:5)
set(gca,'ylim',[1 5],'ytick',1:0.5:5)
set(gca,'GridLineWidth',1,'GridColor',[0.15 0.15 0.15])
axis square
xlabel('Mean response Lovik et al. (2017)')
ylabel('Mean response ChatGPT')
h=findobj('FontName','Helvetica');
set(h,'FontSize',28,'Fontname','Arial')
set(gcf,'WindowState','maximized');print(gcf,'Figure1.eps','-deps','-r600');
set(gcf,'WindowState','maximized');print(gcf,'Figure1.png','-dpng','-r600');
